# -*- coding: utf-8 -*-

from django.db import models
from django.core.validators import MinValueValidator, RegexValidator
from portailuser.models import PortailUser
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.core.mail import send_mail
from portail.settings import ECOBOX_MAIL
from gestion import fields
import random
import string
from imagekit.models import ProcessedImageField

telephone_validateur = RegexValidator(
    regex=r"^0[67]\d{8}$",
    message="Veuillez rentrer un numéro de téléphone valide. 06xxxxxxxx ou 07xxxxxxxx",
)


class Fournisseur(models.Model):
    nom = models.CharField(max_length=100, null=True, unique=True)

    def __str__(self):
        return self.nom


class Ecobox(models.Model):
    nom = models.CharField(max_length=100)
    # La valeur par défaut est présente à cause de l'ajout de l'ensat
    ecole = models.CharField(max_length=10, editable=False, default="n7")
    prix = models.DecimalField(max_digits=5, decimal_places=2, validators=[MinValueValidator(0)])
    description = models.TextField()
    disponible = models.BooleanField(default=True)
    visible = models.BooleanField(default=True)
    photo = ProcessedImageField(upload_to="photos_ecobox_produit/", blank=True)
    fournisseur = models.ForeignKey(Fournisseur, blank=True, null=True, on_delete=models.SET_NULL)
    categorie = models.CharField(max_length=30, blank=True)

    def chaine_prix(self):
        return mark_safe((f"{self.prix:.2f}").replace(".", ",") + " €")

    def chaine_prix_js(self):
        return f"{self.prix:.2f}"

    def __str__(self):
        return self.nom


class PointRetrait(models.Model):
    adresse = models.CharField(max_length=128, null=True, blank=True)
    ecole = models.CharField(max_length=10, editable=False, default="n7")

    def __str__(self):
        return self.adresse


class DateEcobox(models.Model):
    date_livraison = fields.DateField(verbose_name="Date de livraison")
    date_fin_commande = fields.DateTimeField(verbose_name="Date de fin des commandes")
    ecobox_dispo = models.ManyToManyField(Ecobox, verbose_name="Produits disponibles")
    ecole = models.CharField(max_length=10, editable=False, default="n7")
    points_retrait = models.ManyToManyField(
        PointRetrait,
        blank=True,
        verbose_name="Adresses de livraison possibles",
        help_text="(Optionnel)",
    )


def randomkey():
    return "".join(random.SystemRandom().choice(string.ascii_letters + string.digits) for _ in range(128))


class Commande(models.Model):
    date_reservation = fields.DateTimeField(auto_now_add=True, verbose_name="Date de réservation")
    date = models.ForeignKey(DateEcobox, on_delete=models.CASCADE)
    point_retrait = models.ForeignKey(
        PointRetrait, on_delete=models.SET_NULL, null=True, verbose_name="Point de retrait"
    )
    paye = models.BooleanField(default=False, verbose_name="Payée")
    recupere = models.BooleanField(default=False, verbose_name="Récupérée")
    # elements -> [ContenuReservation]

    user = models.ForeignKey(PortailUser, blank=True, null=True, on_delete=models.CASCADE, verbose_name="Client")

    is_register = models.BooleanField(default=True)

    # Ne pas accéder à ces informations en consultation, utiliser les fonctions ci dessous
    # Un client peut-être un élève ou une personne de l'administration
    mail = models.EmailField(blank=True)
    nom = models.CharField(max_length=128, verbose_name="Nom", blank=True)
    prenom = models.CharField(max_length=128, verbose_name="Prenom", blank=True)
    portable = models.CharField(
        max_length=10,
        validators=[telephone_validateur],
        blank=True,
        verbose_name="Téléphone portable",
    )
    hashsecure = models.CharField(max_length=128, default="")

    def prix(self):
        return sum([e.prix * e.quantite for e in self.elements.all()])

    def chaine_prix(self):
        return mark_safe((f"{self.prix():.2f}").replace(".", ",") + " €")

    def client_est_inscrit(self):
        return self.is_register

    def get_mail(self):
        if self.client_est_inscrit():
            return self.user.get_mail_ecole()
        return self.mail

    def get_full_name(self):
        if self.client_est_inscrit():
            return self.user.cn
        return self.nom + " " + self.prenom

    def get_phone_link(self):
        if self.client_est_inscrit():
            return self.user.get_formated_mobile()
        return PortailUser.format_tel(self.portable)

    def get_phone(self):
        if self.client_est_inscrit():
            return self.user.get_mobile()
        return self.portable

    def gen_hash(self):
        self.hashsecure = randomkey()

    def mail_admin(self, request):
        if not self.client_est_inscrit():
            mail_from = ECOBOX_MAIL[str(request.ecole)]
            mail_to = [self.mail]
            sujet_mail = f"Votre Commande d'écobox pour le {self.date.date_livraison}"
            c = {
                "commande": self,
                "url_base": "https:" + request.ecole.get_absolute_url(),
            }
            contenu_mail = render_to_string("ecobox/mail_admin.txt", c)
            contenu_mail_html = render_to_string("ecobox/mail_admin_html.txt", c)
            send_mail(
                sujet_mail,
                contenu_mail,
                mail_from,
                mail_to,
                fail_silently=False,
                html_message=contenu_mail_html,
            )

    def __str__(self):
        return f"Commande de {self.user.get_full_name()}"


class ContenuCommande(models.Model):
    commande = models.ForeignKey(Commande, related_name="elements", on_delete=models.CASCADE)
    ecobox = models.ForeignKey(Ecobox, on_delete=models.CASCADE)
    quantite = models.IntegerField(validators=[MinValueValidator(0)])
    prix = models.DecimalField(max_digits=5, decimal_places=2, validators=[MinValueValidator(0)], default=0)

    def __str__(self):
        return f"{self.quantite} x {self.ecobox}"

    def synchro_prix(self):
        # méthode appelé pour remplir le champ prix
        # Le prix d'une commande ne change pas si on change le prix d'une écobox
        self.prix = self.ecobox.prix
        self.save()

    def chaine_prix(self):
        return mark_safe((f"{self.prix:.2f}").replace(".", ",") + " €")

    def total_prix(self):
        return self.quantite * self.prix


class Blackliste(models.Model):
    personne = models.OneToOneField(PortailUser, on_delete=models.CASCADE)


class BlacklisteMail(models.Model):
    mail = models.EmailField()
